// Witold Kardy EIK - przykad uycia algorytmu stable_sort - ksika telefoniczna
#include <iostream>
#include <algorithm>
#include <vector>
using namespace std;

//klasa przechowujca rekord danych ksiki telefonicznej - imi, nazwisko i nr telefonu
class rekord
{
public:
	string imie, nazwisko, tel; //skadowe przechowujce waciwe dane
	enum tryb {by_imie, by_nazwisko, by_tel}; //moliwe sposoby sortowania
	static tryb comp_mode;//statyczna zmienna przechowujca sposb sortowania
	//konstruktor
	rekord(string im, string nazw, string t):imie(im),nazwisko(nazw),tel(t) {};
	//operator wpisu do strumienia - dla wygody drukowania rekordu
	friend ostream& operator<<(ostream& str,const rekord& source);
	//operator porwnania wykorzystywany przez algorytm stable_sort
	friend bool operator<(const rekord& a, const rekord& b);
	//funkcja porwnujca w sposb odwrotny, podawana jako argument stable_sort
	friend bool comparator(const rekord& a, const rekord& b);
};
rekord::tryb rekord::comp_mode; //statyczna zmienna przechowujca sposb sortowania
//operator porwnania wykorzystywany przez funkcj stable_sort
bool operator<(const rekord& a, const rekord& b)
{
	switch(rekord::comp_mode)//wybr sortowanego pola
	{
		case rekord::by_imie:		if(a.imie.compare(b.imie)<0)return true;		break;
		case rekord::by_nazwisko:	if(a.nazwisko.compare(b.nazwisko)<0)return true;break;
		case rekord::by_tel:		if(a.tel.compare(b.tel)<0)return true;			break;
	}
	return false;
};
//funkcja porwnujca w sposb odwrotny (zamiast operatora domylnego)
//podawana jako trzeci argument algorytmu stable_sort
bool comparator(const rekord& a, const rekord& b)
{
	switch(rekord::comp_mode)//wybr sortowanego pola
	{
		case rekord::by_imie:		if(a.imie.compare(b.imie)>0)return true;		break;
		case rekord::by_nazwisko:	if(a.nazwisko.compare(b.nazwisko)>0)return true;break;
		case rekord::by_tel:		if(a.tel.compare(b.tel)>0)return true;			break;
	}
	return false;
}
//zaprzyjaniony operator wpisu do strumienia uatwiajcy drukowanie rekordu
ostream& operator<<(ostream& str,const rekord& source)
{
	str.width(15);
	str<<left<<source.nazwisko.c_str();
	str.width(15);
	str<<left<<source.imie.c_str();
	str.width(15);
	str<<left<<source.tel.c_str()<<endl;
	return str;
};

//------------------------------------------------------------------------------------------
// ptla gwna
int main () {
  vector<rekord> spis; //wektor wpisw w ksice telefonicznej
  vector<rekord>::iterator iter; //iterator dla wygody

  //wpisujemy jakie rekordy do ksiki w kolejnoci losowej
  spis.push_back(rekord("Zbigniew", "Nowacki", "0-700700700"));
  spis.push_back(rekord("Adam", "Czajkowski", "0202122"));
  spis.push_back(rekord("Antonow", "Nowacki", "0-12345678"));
  spis.push_back(rekord("Adam", "Zolowski", "222222222"));
  spis.push_back(rekord("Jaroslaw", "Kwiatkowski", "0-800100100"));
  spis.push_back(rekord("Sylwester", "Zolowski", "333333333"));
  spis.push_back(rekord("Adam", "Garowy", "997"));
  spis.push_back(rekord("Karol", "Zolowski", "87654321"));
  
  cout<<"Ksiazka telefoniczna - przyklad uzycia algorytmu stable_sort";
  cout<<"\nNieposortowana:\n";
  //wypisujemy zawarto nieposortowanej ksiki
  for(iter=spis.begin(); iter!=spis.end(); ++iter) cout<<*iter;
  
  cout<<"\nPosortowana wedlug imion:\n";
  //ustawiamy sposb sortowania na sortowanie po imionach
  rekord::comp_mode=rekord::by_imie;
  //wykonujemy sortowanie - funkcja sortuje wektor w podanym zakresie 
  //z uyciem domylnego operatora porwnania (operator <)
  stable_sort (spis.begin(), spis.end());
  //wypisujemy zawarto ksiki posortowanej po imionach
  for (iter=spis.begin(); iter!=spis.end(); ++iter) cout << *iter;
  
  cout<<"\nPosortowana wedlug nazwisk:\n";
  //ustawiamy sposb sortowania na sortowanie po nazwiskach
  rekord::comp_mode=rekord::by_nazwisko;
  //wykonujemy sortowanie - ksika zostaje posortowana wzgldem nazwisk
  //ale dla rekordw o tym samym nazwisku utrzymana zostaje kolejno 
  //pozostaa po poprzednim sortowaniu czyli sortowanie po imionach
  //nie wystpuje efekt zamiany kolejnoci elementw rwnych sobie
  //umoliwia to sortowanie po wielu skadowych klasy
  stable_sort (spis.begin(), spis.end());
  //wypisujemy ostatecznie posortowan ksik
  for (iter=spis.begin(); iter!=spis.end(); ++iter) cout << *iter;
  
  cout<<"\nPosortowana wedlug nazwisk malejaco:\n";
  //wykonujemy sortowanie z uyciem drugiej wersji algorytmu stable_sort
  //tym razem zamiast wykorzystywa operator < zdefiniowany w klasie
  //podamy jawnie funkcj uywan do porwnania
  //aby uwidoczni dziaanie tej funkcji odwraca ona sposb dziaania
  //operatora porwnania (dziaa jak poerator >) co umoliwia sortowanie w porzdku malejcym
  //jest to rozwizanie bardziej przejyste ni przypisanie operatorowi < dziaania 
  //waciwego dla operatora >
  //wczeniejsze sortowanie po imionach zostaje cigle utrzymane
  stable_sort (spis.begin(), spis.end(), comparator);
  //wypisujemy ostatecznie posortowan ksik
  for (iter=spis.begin(); iter!=spis.end(); ++iter) cout << *iter;
  cout<<"\nKoniec\n";

  return 0;
}
